@@ -1,4 +1,6 @@
+# -*- coding: utf-8 -*-
 import numpy as np
+
 from qibo import gates
 from qibo.config import log, raise_error
 from qibo.models.circuit import Circuit
@@ -36,7 +38,7 @@ class Grover(object):
         iterative (bool): force the use of the iterative Grover
 
     Example:
-        ::
+        .. testcode::
 
             import numpy as np
             from qibo import gates
@@ -53,9 +55,19 @@ class Grover(object):
             solution, iterations = grover()
     """
 
-    def __init__(self, oracle, superposition_circuit=None, initial_state_circuit=None,
-                 superposition_qubits=None, superposition_size=None, number_solutions=None,
-                 target_amplitude = None, check=None, check_args=(), iterative=False):
+    def __init__(
+        self,
+        oracle,
+        superposition_circuit=None,
+        initial_state_circuit=None,
+        superposition_qubits=None,
+        superposition_size=None,
+        number_solutions=None,
+        target_amplitude=None,
+        check=None,
+        check_args=(),
+        iterative=False,
+    ):
 
         self.oracle = oracle
         self.initial_state_circuit = initial_state_circuit
@@ -64,9 +76,12 @@ class Grover(object):
             self.superposition = superposition_circuit
         else:
             if not superposition_qubits:
-                raise_error(ValueError, "Cannot create Grover model if the "
-                                        "superposition circuit or number of "
-                                        "qubits is not specified.")
+                raise_error(
+                    ValueError,
+                    "Cannot create Grover model if the "
+                    "superposition circuit or number of "
+                    "qubits is not specified.",
+                )
             self.superposition = Circuit(superposition_qubits)
             self.superposition.add([gates.H(i) for i in range(superposition_qubits)])
 
@@ -78,14 +93,16 @@ class Grover(object):
         if superposition_size:
             self.sup_size = superposition_size
         else:
-            self.sup_size = int(2 ** self.sup_qubits)
+            self.sup_size = int(2**self.sup_qubits)
 
         assert oracle.nqubits > self.sup_qubits
 
         self.anc_qubits_sup = self.superposition.nqubits - self.sup_qubits
         self.anc_qubits_ora = self.oracle.nqubits - self.sup_qubits - 1
 
-        self.nqubits = self.sup_qubits + max(self.anc_qubits_sup, self.anc_qubits_ora) + 1
+        self.nqubits = (
+            self.sup_qubits + max(self.anc_qubits_sup, self.anc_qubits_ora) + 1
+        )
 
         self.check = check
         self.check_args = check_args
@@ -94,15 +111,21 @@ class Grover(object):
         self.iterative = iterative
 
         self.space_sup = list(range(self.sup_qubits + self.anc_qubits_sup))
-        self.space_ora = list(range(self.sup_qubits + self.anc_qubits_ora)) + [self.nqubits-1]
+        self.space_ora = list(range(self.sup_qubits + self.anc_qubits_ora)) + [
+            self.nqubits - 1
+        ]
 
     def initialize(self):
         """Initialize the Grover algorithm with the superposition and Grover ancilla."""
         c = Circuit(self.nqubits)
-        c.add(gates.X(self.nqubits-1))
-        c.add(gates.H(self.nqubits-1))
+        c.add(gates.X(self.nqubits - 1))
+        c.add(gates.H(self.nqubits - 1))
         if self.initial_state_circuit:
-            c.add(self.initial_state_circuit.invert().on_qubits(*range(self.initial_state_circuit.nqubits)))
+            c.add(
+                self.initial_state_circuit.invert().on_qubits(
+                    *range(self.initial_state_circuit.nqubits)
+                )
+            )
         c.add(self.superposition.on_qubits(*self.space_sup))
         return c
 
@@ -110,22 +133,30 @@ class Grover(object):
         """Construct the diffusion operator out of the superposition circuit."""
         nqubits = self.superposition.nqubits + 1
         c = Circuit(nqubits)
-        c.add(self.superposition.invert().on_qubits(*range(nqubits-1)))
+        c.add(self.superposition.invert().on_qubits(*range(nqubits - 1)))
         if self.initial_state_circuit:
-            c.add(self.initial_state_circuit.invert().on_qubits(*range(self.initial_state_circuit.nqubits)))
+            c.add(
+                self.initial_state_circuit.invert().on_qubits(
+                    *range(self.initial_state_circuit.nqubits)
+                )
+            )
         c.add([gates.X(i) for i in range(self.sup_qubits)])
-        c.add(gates.X(nqubits-1).controlled_by(*range(self.sup_qubits)))
+        c.add(gates.X(nqubits - 1).controlled_by(*range(self.sup_qubits)))
         c.add([gates.X(i) for i in range(self.sup_qubits)])
         if self.initial_state_circuit:
-            c.add(self.initial_state_circuit.on_qubits(*range(self.initial_state_circuit.nqubits)))
-        c.add(self.superposition.on_qubits(*range(nqubits-1)))
+            c.add(
+                self.initial_state_circuit.on_qubits(
+                    *range(self.initial_state_circuit.nqubits)
+                )
+            )
+        c.add(self.superposition.on_qubits(*range(nqubits - 1)))
         return c
 
     def step(self):
         """Combine oracle and diffusion for a Grover step."""
         c = Circuit(self.nqubits)
         c.add(self.oracle.on_qubits(*self.space_ora))
-        c.add(self.diffusion().on_qubits(*(self.space_sup+[self.nqubits-1])))
+        c.add(self.diffusion().on_qubits(*(self.space_sup + [self.nqubits - 1])))
         return c
 
     def circuit(self, iterations):
@@ -144,17 +175,23 @@ class Grover(object):
         c.add(gates.M(*range(self.sup_qubits)))
         return c
 
-    def iterative_grover(self, lamda_value=6/5):
+    def iterative_grover(self, lamda_value=6 / 5, backend=None):
         """Iterative approach of Grover for when the number of solutions is not known.
 
         Args:
             lamda_value (real): parameter that controls the evolution of the iterative method.
                                 Must be between 1 and 4/3.
+            backend (:class:`qibo.backends.abstract.Backend`): Backend to use for circuit execution.
 
         Returns:
             measured (str): bitstring measured and checked as a valid solution.
             total_iterations (int): number of times the oracle has been called.
         """
+        if backend is None:  # pragma: no cover
+            from qibo.backends import GlobalBackend
+
+            backend = GlobalBackend()
+
         k = 1
         lamda = lamda_value
         total_iterations = 0
@@ -163,16 +200,16 @@ class Grover(object):
             if it != 0:
                 total_iterations += it
                 circuit = self.circuit(it)
-                result = circuit(nshots=1)
+                result = backend.execute_circuit(circuit, nshots=1)
                 measured = result.frequencies(binary=True).most_common(1)[0][0]
                 if self.check(measured, *self.check_args):
                     return measured, total_iterations
             k = min(lamda * k, np.sqrt(self.sup_size))
-            if total_iterations > (9/4) * np.sqrt(self.sup_size):
+            if total_iterations > (9 / 4) * np.sqrt(self.sup_size):
                 log.warning("Too many total iterations, output might not be solution.")
                 return measured, total_iterations
 
-    def execute(self, nshots=100, freq=False, logs=False):
+    def execute(self, nshots=100, freq=False, logs=False, backend=None):
         """Execute Grover's algorithm.
 
         If the number of solutions is given, calculates iterations,
@@ -181,25 +218,34 @@ class Grover(object):
         Args:
             nshots (int): number of shots in order to get the frequencies.
             freq (bool): print the full frequencies after the exact Grover algorithm.
+            backend (:class:`qibo.backends.abstract.Backend`): Backend to use for circuit execution.
 
         Returns:
             solution (str): bitstring (or list of bitstrings) measured as solution of the search.
             iterations (int): number of oracle calls done to reach a solution.
         """
+        if backend is None:  # pragma: no cover
+            from qibo.backends import GlobalBackend
+
+            backend = GlobalBackend()
+
         if (self.num_sol or self.targ_a) and not self.iterative:
             if self.targ_a:
-                it = int(np.pi * (1/self.targ_a) / 4)
+                it = int(np.pi * (1 / self.targ_a) / 4)
             else:
                 it = int(np.pi * np.sqrt(self.sup_size / self.num_sol) / 4)
             circuit = self.circuit(it)
-            result = circuit(nshots=nshots).frequencies(binary=True)
+            result = backend.execute_circuit(circuit, nshots=nshots)
+            result = result.frequencies(binary=True)
             if freq:
                 if logs:
                     log.info("Result of sampling Grover's algorihm")
                     log.info(result)
                 self.frequencies = result
             if logs:
-                log.info(f"Most common states found using Grover's algorithm with {it} iterations:")
+                log.info(
+                    f"Most common states found using Grover's algorithm with {it} iterations:"
+                )
             if self.targ_a:
                 most_common = result.most_common(1)
             else:
@@ -213,22 +259,23 @@ class Grover(object):
                 if logs:
                     if self.check:
                         if self.check(i[0], *self.check_args):
-                            log.info('Solution checked and successful.')
+                            log.info("Solution checked and successful.")
                         else:
-                            log.info('Not a solution of the problem. Something went wrong.')
+                            log.info(
+                                "Not a solution of the problem. Something went wrong."
+                            )
         else:
             if not self.check:
                 raise_error(ValueError, "Check function needed for iterative approach.")
-            measured, total_iterations = self.iterative_grover()
+            measured, total_iterations = self.iterative_grover(backend=backend)
             if logs:
-                log.info('Solution found in an iterative process.')
-                log.info(f'Solution: {measured}')
-                log.info(f'Total Grover iterations taken: {total_iterations}')
+                log.info("Solution found in an iterative process.")
+                log.info(f"Solution: {measured}")
+                log.info(f"Total Grover iterations taken: {total_iterations}")
             self.solution = measured
             self.iterations = total_iterations
         return self.solution, self.iterations
 
-
-    def __call__(self, nshots=100, freq=False, logs=False):
+    def __call__(self, nshots=100, freq=False, logs=False, backend=None):
         """Equivalent to :meth:`qibo.models.grover.Grover.execute`."""
-        return self.execute(nshots=nshots, freq=freq, logs=logs)
+        return self.execute(nshots=nshots, freq=freq, logs=logs, backend=backend)
